from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.error_response import ErrorResponse
from ...models.get_process_identifier_logs_response_200 import GetProcessIdentifierLogsResponse200
from ...types import UNSET, Response, Unset


def _get_kwargs(
    identifier: str,
    *,
    stream: Union[Unset, bool] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["stream"] = stream

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/process/{identifier}/logs",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Optional[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    if response.status_code == 200:
        response_200 = GetProcessIdentifierLogsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 404:
        response_404 = ErrorResponse.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = ErrorResponse.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    identifier: str,
    *,
    client: Union[Client],
    stream: Union[Unset, bool] = UNSET,
) -> Response[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    """Get process logs

     Get the stdout and stderr output of a process

    Args:
        identifier (str):
        stream (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        stream=stream,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    identifier: str,
    *,
    client: Union[Client],
    stream: Union[Unset, bool] = UNSET,
) -> Optional[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    """Get process logs

     Get the stdout and stderr output of a process

    Args:
        identifier (str):
        stream (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, GetProcessIdentifierLogsResponse200]
    """

    return sync_detailed(
        identifier=identifier,
        client=client,
        stream=stream,
    ).parsed


async def asyncio_detailed(
    identifier: str,
    *,
    client: Union[Client],
    stream: Union[Unset, bool] = UNSET,
) -> Response[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    """Get process logs

     Get the stdout and stderr output of a process

    Args:
        identifier (str):
        stream (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        stream=stream,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    identifier: str,
    *,
    client: Union[Client],
    stream: Union[Unset, bool] = UNSET,
) -> Optional[Union[ErrorResponse, GetProcessIdentifierLogsResponse200]]:
    """Get process logs

     Get the stdout and stderr output of a process

    Args:
        identifier (str):
        stream (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, GetProcessIdentifierLogsResponse200]
    """

    return (
        await asyncio_detailed(
            identifier=identifier,
            client=client,
            stream=stream,
        )
    ).parsed



import asyncio
import logging
import random
import string

from blaxel.client.models import Metadata, Port, Runtime, Sandbox, SandboxSpec
from blaxel.common.settings import settings
from blaxel.sandbox.client import client
from blaxel.sandbox.client.models import ProcessRequest
from blaxel.sandbox.sandbox import SandboxInstance

logger = logging.getLogger(__name__)


def random_string(length: int):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length)).lower()


async def create_sandbox(sandbox_name: str):
    # Create sandbox
    image = "blaxel/prod-base:latest"
    logger.info(f"Creating sandbox {sandbox_name} with image {image}")
    sandbox = await SandboxInstance.create(Sandbox(
        metadata=Metadata(name=sandbox_name),
        spec=SandboxSpec(
            runtime=Runtime(
                image=image,
                memory=2048,
                cpu=2,
                ports=[
                    Port(name="sandbox-api", target=8080, protocol="HTTP")
                ]
            )
        )
    ))
    logger.info("Waiting for sandbox to be deployed")
    await sandbox.wait(max_wait=120000, interval=1000)
    logger.info("Sandbox deployed")
    return sandbox

async def run_process_with_retry(sandbox: SandboxInstance, command: str):
    for _ in range(2):
        try:
            # new_client = client.with_base_url(sandbox.process.url).with_headers(settings.headers)
            # response = await new_client.get_async_httpx_client().get(f"/health")
            # print(response.text)
            dir = await sandbox.fs.ls("/")
            print(dir)
            process = await sandbox.process.exec(ProcessRequest(name="test", command=command))
            print(f"Process {process.name} started with pid {process.pid}")
            return process
        except Exception as e:
            print("Failed to run process, retrying...", e)
            await asyncio.sleep(1)

async def main():
    sandbox_name = random_string(10)
    try:
        print(f"Creating sandbox {sandbox_name}")
        sandbox = await create_sandbox(sandbox_name)
        sandbox = await SandboxInstance.get(sandbox_name)
        await run_process_with_retry(sandbox, 'pwd')
        await SandboxInstance.delete(sandbox_name)
    except Exception as e:
        print(e)
    finally:
        await SandboxInstance.delete(sandbox_name)

    sandbox_name = random_string(10)
    try:
        print(f"Creating sandbox {sandbox_name}")
        sandbox = await create_sandbox(sandbox_name)
        sandbox = await SandboxInstance.get(sandbox_name)
        await run_process_with_retry(sandbox, 'pwd')
        await SandboxInstance.delete(sandbox_name)
    except Exception as e:
        print(e)
    finally:
        await SandboxInstance.delete(sandbox_name)


if __name__ == "__main__":
    asyncio.run(main())
